@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2017
+# Copyright (C) 2015-2018
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -27,9 +27,7 @@ from abc import ABCMeta
 
 
 class TelegramObject(object):
-    """
-    Base class for most telegram objects.
-    """
+    """Base class for most telegram objects."""
 
     __metaclass__ = ABCMeta
     _id_attrs = ()
@@ -53,6 +51,7 @@ class TelegramObject(object):
         """
         Returns:
             :obj:`str`
+
         """
 
         return json.dumps(self.to_dict())
@@ -71,6 +70,8 @@ class TelegramObject(object):
                 else:
                     data[key] = value
 
+        if data.get('from_user'):
+            data['from'] = data.pop('from_user', None)
         return data
 
     def __eq__(self, other):
